class AudioInstance:
    def __init__(self, filename, audio, speaker_gender):
        self.filename = filename
        self.audio = audio
        self.gender = speaker_gender
        assert self.gender.lower().strip() in ["m", "f"]

    def speaker_male(self):
        return self.gender.lower().strip() == "m"



#!/usr/bin/python
# coding: utf-8
import pickle
import system
import numpy as np
from sklearn import metrics
from pandas import DataFrame


def classification_results_for(session_id, subject, results_folder, experiment):
    filename = results_folder + "/{}_s{}_{}.p".format(experiment, session_id, subject)
    if system.exists(filename):
        return pickle.load(open(filename, "rb"))
    return None


def analize(results, experiment, measures):
    result = results[experiment]
    processed_result = {}
    for (measure_name, measure_function) in measures:
        measure_result, support = results_extractor(result, measure_function, permutation=False)
        processed_result[measure_name] = measure_result

        permutation_keys = [k for k in list(results.keys()) if k.endswith("_permutation")]
        if len(permutation_keys) > 1:
            perm_results = [results_extractor(results[p], measure_function, permutation=True)[0] for p in permutation_keys]
            # processed_result[measure_name + "_perc_05"] = np.percentile(perm_results, 05)
            # processed_result[measure_name + "_perc_50"] = np.percentile(perm_results, 50)
            # processed_result[measure_name + "_perc_95"] = np.percentile(perm_results, 95)
            processed_result[measure_name + "_p_val"] = len([p for p in perm_results if p >= measure_result]) * 1.0 / (len(perm_results))

    total_support = support
    processed_result.update({
                            "experiment": experiment,
                            "support": total_support,
                            })
    return processed_result


def compare_subjects(sessions, results_folder, measures, experiments, warnings=True):
    res = []
    for (session_id, subject) in sessions:
        try:
            subject_session = (session_id, subject)
            for experiment in experiments:
                results = classification_results_for(session_id, subject, results_folder, experiment)
                if not results:
                    print("no results found for", (session_id, subject, results_folder, experiment))
                    continue
                row = analize(results, experiment, measures)
                row.update({"session": subject_session})
                res.append(row)
        except:
            continue

    return DataFrame(res)


def auc_fn(actual, predicted_probabilities, categories):
    predicted_proba = [p[1] for p in predicted_probabilities]
    return metrics.roc_auc_score(actual, predicted_proba)


def accuracy_fn(actual, predicted_probabilities, categories):
    predicted_proba = [p[1] for p in predicted_probabilities]
    predicted = [1 if p > 0.5 else 0 for p in predicted_proba]
    return metrics.accuracy_score(actual, predicted)


def results_extractor(results, measure, permutation):
    categories = results["categories"]
    classifier_results = results["results"][list(results["results"].keys())[0]]

    supports = []
    all_actuals = []
    all_predicted_probabilities = []
    all_feature_importances = []

    for i in list(classifier_results.keys()):
        res_fold_i = classifier_results[i]
        actual = res_fold_i["actual"]
        predicted_probabilities = res_fold_i["predicted_probabilities"]
        all_actuals.extend(actual)

        if not permutation:
            all_feature_importances.append(res_fold_i["classifier_weights"])

        all_predicted_probabilities.extend(predicted_probabilities)
        supports.append([sum(actual == c) for c in categories])

    measure_result = measure(all_actuals, all_predicted_probabilities, categories)
    support = np.sum(supports, axis=0)

    return measure_result, list(zip(categories, support))


def feature_importances(results):
    classifier_results = results["results"][list(results["results"].keys())[0]]
    all_feature_importances = []

    for i in list(classifier_results.keys()):
        res_fold_i = classifier_results[i]
        all_feature_importances.append(res_fold_i["classifier_weights"])

    feature_importances_by_folds = np.array(all_feature_importances)
    return feature_importances_by_folds.mean(axis=0), feature_importances_by_folds.std(axis=0)



# coding: utf-8
import feature_extraction
import collections
import system
import arff
import numpy as np
import scipy.stats
import time
import os.path
import inspect


class AudioExtractor(feature_extraction.FeatureExtractor):
    pass


class AcousticsExtractor(AudioExtractor):
    # ; Duration
    # ; Entonación (F0 slope)
    # ; Intensity Level (Mean Intensity)
    # ; Pitch Level (Mean Pitch)
    # ; Voice Quality (Jitter)
    # ; Voice Quality (Shimmer)
    # ; Voice Quality (NHR)

    def __init__(self, params):
        self.pitch_config = params["pitch_config"]
        self.intensity_config = params["intensity_config"]
        # self.voice_quality_config = params["voice_quality_config"]
        self.temp_folder = "/tmp/opensmile_arffs/"
        self.extended_features = params["extended_features"]
        self.extract_on_last_seconds = params["extract_on_last_seconds"]
        system.mkdir_p(self.temp_folder)

    def extract(self, instance):
        last_seconds_values = self.extract_on_last_seconds

        duration = instance.audio.duration_seconds
        times_pitch, pitch = self.extract_pitch(instance)
        times_intensities, intensities = self.extract_intensity(instance)

        all_values = {}

        all_values["f0_slope"] = collections.defaultdict(lambda: np.nan)

        all_values["mean_pitch"] = collections.defaultdict(lambda: np.nan)
        all_values["mean_intensity"] = collections.defaultdict(lambda: np.nan)
        all_values["mean_shimmer"] = collections.defaultdict(lambda: np.nan)
        all_values["mean_jitter"] = collections.defaultdict(lambda: np.nan)
        all_values["mean_nhr"] = collections.defaultdict(lambda: np.nan)

        for last_secs in last_seconds_values:
            if last_secs == "all":
                last_secs = duration
                label = "all"
            else:
                label = last_secs

            if duration < last_secs:
                continue

            jitter, shimmer, nhr = self.extract_voice_quality(instance, start=duration - last_secs, end=duration)
            all_values["mean_jitter"][label] = jitter
            all_values["mean_shimmer"][label] = shimmer
            all_values["mean_nhr"][label] = nhr

            for times, values, feat_name in zip(
                [times_pitch, times_intensities],
                [pitch, intensities],
                ["pitch", "intensity"]
            ):

                indices = (times >= duration - last_secs)
                all_values["mean_" + feat_name][label] = np.mean(values[indices])

            indices = (times_pitch >= duration - last_secs)
            if len(times_pitch[indices]) > 3:
                all_values["f0_slope"][label] = scipy.stats.linregress(times_pitch[indices], pitch[indices])[0]  # [0] => slope (m)

        feat = dict(ipu_duration=duration)

        for last_secs in last_seconds_values:
            if last_secs == "all":
                in_ms = "all"
            else:
                in_ms = "last_{}ms".format(int(last_secs * 1000))

            for feat_name in ["f0_slope", "mean_pitch", "mean_intensity", "mean_jitter", "mean_shimmer", "mean_nhr"]:
                feat["{}_{}".format(feat_name, in_ms)] = all_values[feat_name][last_secs]

        if self.extended_features:
            feat["pitch"] = pitch
            feat["times_pitch"] = times_pitch

            feat["intensities"] = intensities
            feat["times_intensities"] = times_intensities

        return feat

    def extract_pitch(self, instance):
        data = self.call_opensmile_script(self.pitch_config, instance.filename)

        times = self.get_column(data, "frameTime")
        pitch = self.get_column(data, "F0final_sma")

        # filter 0s
        times = times[pitch != 0]
        pitch = pitch[pitch != 0]

        return times, pitch

    def extract_intensity(self, instance):
        data = self.call_opensmile_script(self.intensity_config, instance.filename)

        times = self.get_column(data, "frameTime")
        intensities = self.get_column(data, "pcm_intensity_sma")

        times = times[intensities != 0]
        intensities = intensities[intensities != 0]
        intensities_in_dbs = 10 * np.log10(intensities / 10e-14)  # TODO: revisar using I0 = 10 −6 as defined in the opensmile book
        return times, intensities_in_dbs

    def extract_voice_quality(self, instance, start, end):
        min_pitch = 50 if instance.speaker_male() else 75
        max_pitch = 300 if instance.speaker_male() else 500

        script_path = os.path.dirname(inspect.getfile(inspect.currentframe())) + "/voice-analysis.praat"
        try:
            script_path = os.path.dirname(inspect.getfile(inspect.currentframe())) + "/voice-analysis-voiced.praat"
            output = system.run_external_command("praat {}".format(script_path), non_named_params=[instance.filename, start, end, min_pitch, max_pitch])
            values = dict([o.split(":") for o in output.split()])
            jitter = float(values["sound_voiced_local_jitter"]) if "undefined" not in values["sound_voiced_local_jitter"] else np.nan
            shimmer = float(values["sound_voiced_local_shimmer"]) if "undefined" not in values["sound_voiced_local_shimmer"] else np.nan
            nhr = float(values["noise_to_harmonics_ratio"]) if "undefined" not in values["noise_to_harmonics_ratio"] else np.nan
        except:
            print("No voiced frames on range {}-{}".format(start, end))
            jitter = np.nan
            shimmer = np.nan
            nhr = np.nan

        return jitter, shimmer, nhr

        # data = self.call_opensmile_script(self.voice_quality_config, instance.filename)
        # times = self.get_column(data, "frameTime")

        # jitter = self.get_column(data, "jitterLocal")
        # times_jitter = times[jitter != 0]
        # jitter = jitter[jitter != 0]
        #
        # shimmer = self.get_column(data, "shimmerLocal")
        # times_shimmer = times[shimmer != 0]
        # shimmer = shimmer[shimmer != 0]
        #
        # nhr = self.get_column(data, "logHNR")
        # times_nhr = times[nhr != 0]
        # nhr = nhr[nhr != 0]

        # return times_jitter, jitter, times_shimmer, shimmer, times_nhr, nhr

    def call_opensmile_script(self, config, filename):
        timestamp = time.time()
        temp_output = "{}/{}{}.arff".format(self.temp_folder, timestamp, os.path.basename(filename))
        data = {}
        system.run_command("SMILExtract -C {} -I {} -l 0 -arffoutput {} -appendarff 0".format(config, filename, temp_output))
        data = arff.load(open(temp_output, 'rb'))
        system.rm(temp_output)
        return data

    def get_column(self, data, column_name):
        column_id = [a[0] for a in data["attributes"]].index(column_name)
        values = np.array([float(d[column_id]) for d in data["data"]])
        return values



from .equality import EqualityMixin


class Interval(EqualityMixin):
    def __init__(self, start, end, value):
        self.start = float(start)
        self.end = float(end)
        self.value = value
        self.duration = self.end - self.start

    def __iter__(self):
        yield (self.start, self.end, self.value)

    def __repr__(self):
        return "I({a}, {b}, {c} [{d}])".format(a=self.start, b=self.end, c=self.value, d=self.duration)



#!/usr/bin/python
# coding: utf-8


class FeatureExtractor(object):
    def extract(self, instance):
        raise NotImplementedError("{} must implement the 'extract' method".format(self))

    def __init__(self, params):
        raise NotImplementedError("{} must implement the '__init__' method".format(self))

    def batch_extract(self, instances_list):
        raise NotImplementedError("{} must implement the 'extract_batch' method".format(self))

import eeg_feature_extraction
import audio_feature_extraction


def create(**params):
    if params["extraction_method"] == "raw":
        return eeg_feature_extraction.RawExtractor(params)
    elif params["extraction_method"] == "freqs":
        return eeg_feature_extraction.FreqExtractor(params)
    elif params["extraction_method"] == "trash":
        return eeg_feature_extraction.TrashExtractor(params)
    elif params["extraction_method"] == "window_mean":
        return eeg_feature_extraction.WindowedExtractor(params)
    elif params["extraction_method"] == "wavelets":
        return eeg_feature_extraction.WaveletsExtractor(params)
    elif params["extraction_method"] == "opensmile":
        return audio_feature_extraction.AcousticsExtractor(params)



#!/usr/bin/python
# coding: utf-8

from . import utils
from . import data_import
import numpy as np


# Specific data-loading for this project
def set_file_for(session_id, speaker_id, condition, set_files_folder):
    return set_files_folder + "/set-trimmed-trials-epoched-merged-FiltDur-{}/s{}-{}-Deci-Filter-Trim-ICA-Pruned-Epoched-Merged.set".format(condition, session_id, speaker_id)


def data_for_subject(session_id, speaker_id, conditions, corpora_folder, dummy_test=False, verbose=True):
    # En la siguiente sección se obtienen los datos directo de los archivos .set y .fdt y se separan en datos de
    # desarrollo y datos de test. Para esto, por sujeto, por condición, se seleccionan 4/5 de los epochs (de manera aleatoria)
    # para ser utilizados en desarrollo y el quinto restante para resultados finales.

    development_data = {}
    development_epoch_ids = []
    control_data = {}

    for condition in conditions:
        data_for_condition_for_subject = from_eeglab(session_id, speaker_id, condition, corpora_folder, verbose=verbose)  # channels x samples x epochs
        if dummy_test:
            print("generating random data")
            data_for_condition_for_subject = np.random.random(data_for_condition_for_subject.shape) + (conditions.index(condition) * 20)

        n_epochs = data_for_condition_for_subject.shape[2]
        # development_ids, control_ids = split_dev_test_sets(np.arange(n_epochs))
        development_ids = np.arange(n_epochs)
        development_epoch_ids.extend([(condition, i) for i in development_ids])

        development_data_for_condition = data_for_condition_for_subject[:, :, development_ids]
        # control_data_for_condition = data_for_condition_for_subject[:, :, control_ids]

        utils.extend_dict(development_data, condition, development_data_for_condition)
        # extend_dict(control_data, condition, control_data_for_condition)

    return (development_data, control_data, development_epoch_ids)


def from_eeglab(session_id, speaker_id, condition, set_files_folder, verbose=True):
    set_filename = set_file_for(session_id, speaker_id, condition, set_files_folder)
    info, data = data_import.from_eeglab_epochs_setfile(set_filename)
    if verbose:
        print("{}-{}: {} ({} x {} x {}) (channels x samples x epochs)".format(session_id, speaker_id, condition, info["nchan"], data.shape[1], data.shape[2]))

    return data


def split_dev_test_sets(epoch_ids):
    epoch_ids_partition = np.array_split(epoch_ids, 5)

    development_ids = np.concatenate(epoch_ids_partition[0:4])
    validation_ids = epoch_ids_partition[4]
    return development_ids, validation_ids



import numpy as np


def handle_nans(nan_action, X, features_names):
    nan_rows = np.isnan(X).any(axis=1)

    if nan_action == "none" and sum(nan_rows) != 0:
        raise Exception("Error, data contains {} of {} rows with NaN, set the nan_action".format(sum(nan_rows), len(nan_rows)))

    if nan_action == "remove":
        nan_rows = np.isnan(X).any(axis=1)
        X = X[~nan_rows, :]
        print("FILTERNING NaN rows")
        deleted_rows = nan_rows

    if nan_action in ["replace_by_zero", "replace_by_mean", "replace_by_median"]:
        X = np.hstack([X, np.isnan(X) * 1.0])
        features_names = features_names + ["was_nan" + f for f in features_names]

    if nan_action == "replace_by_zero":
        print("converting {} NaN to 0s".format(np.count_nonzero(~np.isnan(X))))
        X = np.nan_to_num(X)
        deleted_rows = np.array([])

    if nan_action == "replace_by_mean":
        col_mean = np.nanmean(X, axis=0)
        inds = np.where(np.isnan(X))
        X[inds] = np.take(col_mean, inds[1])
        deleted_rows = np.array([])

    if nan_action == "replace_by_median":
        col_mean = np.nanmedian(X, axis=0)
        inds = np.where(np.isnan(X))
        X[inds] = np.take(col_mean, inds[1])
        deleted_rows = np.array([])

    return X, deleted_rows, features_names



#!/usr/bin/python
# coding: utf-8
import matplotlib.pyplot as plt
import math

import matplotlib.patches as patches
import numpy as np
from sklearn import metrics
import seaborn as sns
import signal_processing
import mne
from IPython.display import display


def plot_roc_curve(exp_results, roc_title, folds, classifier, ax, permutation, fontsize=30):
    classifier_results = exp_results["results"][classifier]
    all_actuals = []
    all_probs = []

    for i in list(classifier_results.keys()):
        actual = exp_results["results"][classifier][i]["actual"]
        predicted_probabilities = exp_results["results"][classifier][i]["predicted_probabilities"]

        # Compute ROC curve and area the curve
        probs = predicted_probabilities[:, 1]
        all_actuals.extend(actual)
        all_probs.extend(probs)

    fpr, tpr, thresholds = metrics.roc_curve(all_actuals, all_probs)
    roc_auc = metrics.auc(fpr, tpr)

    if permutation:
        ax.plot(fpr, tpr, "b-", alpha=0.5, lw=0.3)
    else:
        ax.plot([0, 1], [0, 1], '--', lw=6, color="k", label='Chance')
        ax.plot(fpr, tpr, "g-", alpha=1, lw=8, label='ROC (area = {})'.format("{:.3f}".format(roc_auc)))
        ax.set_xlim([-0.05, 1.05])
        ax.set_ylim([-0.05, 1.05])
        plt.tick_params(axis='both', which='major', labelsize=30)
        ax.set_xlabel('False Positive Rate', fontsize=fontsize)
        ax.set_ylabel('True Positive Rate', fontsize=fontsize)


def subject_table(data):
    table = data.reindex_axis(["session",
                               "experiment",
                               "auc",
                               "auc_p_val",
                               "acc",
                               "acc_p_val",
                               "support",
                               ], axis=1)
    if not table.empty:
        s = table.style\
                 .applymap(_set_color, subset=["auc_p_val", "acc_p_val"])\
                 .applymap(_bold, subset=["session"])
        display(s)


def _set_color(val):
    if math.isnan(val):
        color = "rgba(255, 255, 255, 1)"
    elif val < 0.05:
        color = "rgba(216, 246, 206, 0.5)"
    elif val <= 0.1:
        color = "rgba(242, 245, 169, 0.5)"
    else:
        color = "rgba(245, 169, 169, 0.3)"
    return 'background-color: %s;' % color


def _bold(val):
    return 'font-weight: bold'


def _bar(val):
    if val > 0.6:
        return _bar_color(val, "green")
    elif val > 0.4:
        return _bar_color(val, "yellow")
    else:
        return _bar_color(val, "red")


def _bar_2(val):
    return _bar_color(val, "blue")


def _bar_color(val, color):
        base = 'width: 10em; height: 80%;'
        attrs = (base + 'background: linear-gradient(90deg,rgba{c} {w}%, '
                        'transparent 0%)')

        color = sns.light_palette(color, as_cmap=True)(val)
        c = (int(color[0] * 255), int(color[1] * 255), int(color[2] * 255), 0.3)
        return attrs.format(c=c, w=val * 100)


def plot_epochs_average(data, y_lim, tmin, window, freq, marks, epochs_max_number, ax):
    t0, tf = window
    t0_frame = signal_processing.frame_at(t0, freq, tmin)
    tf_frame = signal_processing.frame_at(tf, freq, tmin)

    epochs_mean = data[:, t0_frame:tf_frame, 0:epochs_max_number].mean(2)

    samples = epochs_mean.shape[1]

    for channel in range(0, epochs_mean.shape[0]):
        c_plt, = ax.plot([t0 + (s / freq) for s in range(0, samples)], epochs_mean[channel], "k", alpha=0.3, label="each channel")

    c2_plt, = ax.plot([t0 + (s / freq) for s in range(0, samples)], epochs_mean.mean(0), "r", linewidth=2.0, label="channels avg")
    set_plot(plt, y_lim, window, t0, tf, marks, ax)
    rect = draw_rectangle(plt, window, ax)

    ax.legend(handles=[c_plt, c2_plt, rect])


def plot_epochs_comparition(data_dict, y_lim, tmin, window, freq, marks, epochs_max_number):
    t0, tf = window
    t0_frame = signal_processing.frame_at(t0, freq, tmin)
    tf_frame = signal_processing.frame_at(tf, freq, tmin)
    plt.figure()

    for condition in list(data_dict.keys()):
        epochs_mean = data_dict[condition][:, t0_frame:tf_frame, 0:epochs_max_number].mean(2)
        samples = epochs_mean.shape[1]

        plt.plot([t0 + (s / freq) for s in range(0, samples)], epochs_mean.mean(0), linewidth=2.0, label=condition)

    set_plot(plt, y_lim, window, t0, tf, marks, plt)
    draw_rectangle(plt, window)
    plt.legend()


def set_plot(plt, y_lim, window, t0, tf, marks, ax):
    plt.ylim(y_lim)
    plt.xlim(window)
    # plt.xticks([x/1000.0 for x in range(-2000, 101, 100) if (x/1000.0)>=t0 and (x/1000.0)<=tf])
    ax.axvline(marks[0])
    ax.axvline(marks[1])


def draw_rectangle(plt, window, ax=None):
    if not ax:
        ax = plt.gca()
    rect = patches.Rectangle((window[0], -40), width=-window[0] - 0.4, height=80,
                             color='grey',
                             alpha=0.5)
    ax.add_patch(rect)

    rect = patches.Rectangle((0, -40), width=window[1], height=80,
                             color='grey',
                             alpha=0.5)
    ax.add_patch(rect)

    rect = patches.Rectangle((-0.4, -40), width=0.4, height=80,
                             fill=None, label="stimulus")
    ax.add_patch(rect)
    return rect


def barplot(session, data, measure):
    plt.figure()

    sns.set(style="white", context="talk", font_scale=1.0)
    sns.despine(bottom=True)
    no_perumutations = data[not data.experiment.str.endswith("permutation")]
    session_data = no_perumutations[no_perumutations.session == session]

    x = list(session_data.experiment)
    y = list(session_data[measure])
    hue = list(session_data.extraction_method)
    permutations = list(session_data[measure + "_permutation"])
    supports = list(session_data.support)

    bar = sns.barplot(x=x,
                      y=y,
                      palette="Set1",
                      hue=hue,
                      data=session_data)

    rectangles = []
    for i, p in enumerate(bar.patches):
        height = p.get_height()
        xmin = p.get_x()
        width = p.get_width()
        bar.text(xmin, height + 0.01, '%1.2f' % (y[i]), fontsize=14)
        rectangles.append(
            patches.Rectangle(
                (xmin, permutations[i]),   # (x,y)
                width,          # width
                0.01,          # height
                color="black",
            )
        )
        bar.text(p.get_x() + 0.1, 0.1, "N={} ".format(supports[i]), rotation=90, fontsize=14, backgroundcolor="w")

    for rectangle in rectangles:
        bar.add_patch(rectangle)
    plt.ylabel("AUC")
    plt.ylim([0, 1])
    plt.title("Session {}".format(session))
    plt.tight_layout()


def feature_importances_by_window_size(df, title):
    # https://gist.github.com/wmvanvliet/6d7c78ea329d4e9e1217
    gr = sns.stripplot(x="starting_time", y="feature_importances_folds_mean", data=df, hue="window_size", palette="Set2")
    gr.set_xticklabels(gr.get_xticklabels(), rotation=90)
    gr.set_title(title)
    plt.show()
    return gr.get_figure()


def feature_importances_topomap(df, window_sizes, freq):
    montage = mne.channels.read_montage('/home/pbrusco/projects/montages/LNI.sfp')

    vmin = df.feature_importances_folds_mean.min()
    vmax = df.feature_importances_folds_mean.max()
    l = mne.channels.make_eeg_layout(mne.create_info(montage.ch_names, freq, ch_types="eeg", montage=montage))
    for (cmap, window_size, (rows, cols)) in zip(["BuGn", "Oranges", "Blues", "Purples"], window_sizes, [(4, 4)]):  # , (3, 4), (4, 2), (1, 3)
        display(window_size)
        starting_samples = sorted(set(df[df["window_size"] == window_size].starting_sample))
        fig, axes = plt.subplots(rows, cols, figsize=(15, 15))
        # fig.subplots_adjust(hspace=.5)
        axes = axes.flatten()
        [ax.axis('off') for ax in axes]
        for sample, ax in zip(starting_samples, axes):
            sample_data = df[(df["starting_sample"] == sample) & (df["window_size"] == window_size)]
            t = int(float(list(sample_data.starting_time)[0]) * 100) * 10

            values = np.array(sample_data.groupby("channel")["feature_importances_folds_mean"].mean())
            # mne.viz.plot_topomap(values, l.pos[:, 0:2], axes=ax, show_names=True, names=montage.ch_names, outlines="skirt", show=False)
            mne.viz.plot_topomap(values, l.pos[:, 0:2], vmin=vmin, vmax=vmax, axes=ax, show_names=False, names=montage.ch_names, show=False, cmap=cmap, )
            # plt.subplots_adjust(left=0.01, right=0.99, bottom=0.01, top=0.88)
            ax.set_title("{} ms".format(t), fontsize=35)
        fig.tight_layout()
        return fig



from .equality import EqualityMixin


class Turn(EqualityMixin):
    def __init__(self, start, end, intervals):
        self.start = float(start)
        self.end = float(end)
        self.duration = self.end - self.start
        self.intervals = intervals

    def __repr__(self):
        return "T {a} to {b} with: {c} \n".format(a=self.start, b=self.end, c=self.intervals)


def next_ipu(vad):
    return vad.next("1")


def next_silence(vad):
    return vad.next("0")


def is_ipu(interval):
    return interval.value == "1"


def is_silence(interval):
    return interval.value == "0"


def intervals_between(t0, t1, vad):

    return [interval for interval in vad if interval.start <= t1 and interval.end >= t0]


def next_turn(vad, speaker2_vad):
    intervals = []

    following_ipu = next_ipu(vad)
    if not following_ipu:
        return None

    start = following_ipu.start
    intervals.append(following_ipu)

    while True:
        if not vad.has_next():
            return Turn(start, vad.current().end, intervals)

        next_interval = next(vad)

        if is_silence(next_interval):
            overlaped_intervals = intervals_between(next_interval.start, next_interval.end, speaker2_vad)

            if any(map(is_ipu, overlaped_intervals)):
                end = next_interval.start
                return Turn(start, end, intervals)

        intervals.append(next_interval)


def turns(vad, speaker2_vad):
    turns = []
    while True:
        turn = next_turn(vad, speaker2_vad)
        if not turn:
            return turns
        else:
            turns.append(turn)



#!/usr/bin/python
# coding: utf-8

import mne


def from_eeglab_epochs_setfile(set_filename):
    data_mne = mne.io.read_epochs_eeglab(set_filename, events=None, event_id=None)
    return data_mne.info, data_mne.get_data().transpose((1, 2, 0)) * 1e6



from pydub import AudioSegment
import system


def load_wav(filename):
    return AudioSegment.from_wav(filename)


def wav_duration(filename):
    # in seconds
    dur = system.run_command("soxi -D {}".format(filename), verbose=False)
    dur.strip()
    return float(dur)


def sliding_window(sequence, window_size_in_frames, step_in_frames, time_limits, t0, freq, debugging=False):
    frame_duration = 1.0 / freq
    sec_length = sequence.shape[1]
    num_of_chunks = ((sec_length - window_size_in_frames) / step_in_frames) + 1
    ti, tf = time_limits

    for i in range(0, num_of_chunks * step_in_frames, step_in_frames):
        frame_initial_time, frame_end_time = frame_time_limits(i, freq, t0, window_size_in_frames)

        if frame_initial_time < ti or frame_end_time > tf:
            continue
        if debugging:
            step_in_secs = step_in_frames * frame_duration
            print("frame index:", i, "from:", frame_initial_time, "to:", frame_end_time, "next should be:", frame_initial_time + step_in_secs, "step:", step_in_secs)
        yield i, sequence[:, i:i + window_size_in_frames]


def time_sliding_window(sequence, window_size_in_secs, step_in_secs, time_limits, t0, freq, debugging=False):
    frame_duration = 1.0 / freq
    window_size_in_frames = int(window_size_in_secs / frame_duration)  # for example 100 ms = 51 frames aprox if freq = 512.0
    step_in_frames = int(step_in_secs / frame_duration)
    return sliding_window(sequence, window_size_in_frames, step_in_frames, time_limits, t0, freq, debugging)


def interpolate(x, y, interpolator, deg):
    return interpolator(x, y, deg=deg)


def frame_time_limits(sample_number, freq, t0, window_size_in_frames):
    frame_duration = 1.0 / freq
    window_size_in_secs = window_size_in_frames * frame_duration
    frame_initial_time = t0 + frame_duration * sample_number
    frame_end_time = frame_initial_time + window_size_in_secs
    return frame_initial_time, frame_end_time


def frame_at(t, freq, tmin):
    relative_time = t - tmin
    return int(relative_time * freq)


def extract_mfccss(filename):
    # run "SMILExtract -C config/MFCC12_E_D_A.conf -I input.wav -O output.mfcc.htk"
    pass



#!/usr/bin/python
# coding: utf-8
import sklearn.cross_validation
from sklearn.svm import SVC
from sklearn.ensemble import RandomForestClassifier
from . import utils


class ClassificationHelper(object):
    def __init__(self, config, classes=[0, 1], classifier=None, seed=1234):
        if classifier:
            self.classifier = classifier
        else:
            self.classifier = self._build_classifier_from(config, seed=seed)
            print("Using {} classifier".format(self.classifier))

        self.classifier_name = config["classifier_name"]
        self.classes = classes

    def _build_classifier_from(self, config, seed=1234):
        classifiers = {"SVM": SVC(kernel="linear", probability=True, C=1, class_weight='balanced'),  # Balanced da pesos según cantidad de instancias
                       "SVMG": SVC(kernel='rbf', probability=True, gamma=0.7, C=1, class_weight='balanced'),
                       "RF10": RandomForestClassifier(n_estimators=10, criterion='gini', n_jobs=-1, random_state=seed, class_weight='balanced'),
                       "RF100": RandomForestClassifier(n_estimators=100, criterion='gini', n_jobs=-1, random_state=seed, class_weight='balanced'),
                       "RF1000": RandomForestClassifier(n_estimators=1000, criterion='gini', n_jobs=-1, random_state=seed, class_weight='balanced'),
                       }

        return classifiers[config["classifier_name"]]

    def classification_probas(self, X_train, y_train, X_test, seed=1234):
        self.classifier.fit(X_train, y_train)
        return self.classifier.predict_proba(X_test)

    def one_speaker_out_cross_validation(self, X, y, speakers, save_weights, verbose=True, seed=1234):
        folds = sklearn.cross_validation.LeaveOneLabelOut(speakers)
        results = {}
        results[self.classifier_name] = {}

        for i, (train_index, test_index) in enumerate(folds):
            if verbose:
                utils.print_inline("speaker: {} of {} for {}".format(i + 1, len(set(speakers)), self.classifier_name))
            X_train, X_test = X[train_index], X[test_index]
            y_train, y_test = y[train_index], y[test_index]

            results[self.classifier_name][i] = self._classify(X_train, X_test, y_train, save_weights)
            results[self.classifier_name][i]["actual"] = y_test
            results[self.classifier_name][i]["y_ids"] = test_index

        return {"categories": self.classes, "results": results, "y": y}

    def k_fold_cross_validation(self, X, y, n_folds, save_weights, verbose=True, seed=1234, subsample=False):
        counts = [sum(y == c) for c in self.classes]

        if verbose:
            print("running classifiers for", list(zip(self.classes, counts)))
        folds = sklearn.cross_validation.StratifiedKFold(y, n_folds=n_folds, random_state=seed)

        results = {}
        results[self.classifier_name] = {}

        for i, (train_index, test_index) in enumerate(folds):
            if verbose:
                utils.print_inline("K-fold: {} of {} for {}".format(i + 1, n_folds, self.classifier_name))
            X_train, X_test = X[train_index], X[test_index]
            y_train, y_test = y[train_index], y[test_index]

            if subsample:
                sub_ids = utils.subsample_ids(y_train)
                X_train = X_train[sub_ids]
                y_train = y_train[sub_ids]

            results[self.classifier_name][i] = self._classify(X_train, X_test, y_train, save_weights)
            results[self.classifier_name][i]["actual"] = y_test

        return {"categories": self.classes, "results": results, "y": y}

    def _classify(self, X_train, X_test, y_train, save_weights):
        self.classifier.fit(X_train, y_train)
        y_pred_probabilities = self.classifier.predict_proba(X_test)
        res = {"predicted_probabilities": y_pred_probabilities}

        if save_weights:
            try:
                feature_importances = self.classifier.feature_importances_
            except:
                try:
                    feature_importances = self.classifier.coef_
                except:
                    feature_importances = None
            res.update({"classifier_weights": feature_importances})
        return res
